//
//  MIDIOPlugIn.m
//  MIDIO
//
//  Created by Dustin O'Connor on 3/23/12.
//  Copyright (c) 2012 d0cut0uch. All rights reserved.
//

// It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering
#import <OpenGL/CGLMacro.h>

#import "MIDIOPlugIn.h"

#define	kQCPlugIn_Name				@"MIDIO by Dustin O'Connor"
#define	kQCPlugIn_Description		@"chord plays chord from a given root note. scale lets you choose between a particular scale. index is the note in a one octave scale to play. note is your root midi note, set this to a midi note number, trigger you must trigger an on followed by an off, velocity is the velocity the notes are played, channel is the midi channel in which you want to play on and device is the index number of the midi device you want to play out on. (use midi info plug to see names of devices) typical uses of this plugin would be inside an iterator."

#pragma mark utility functions
static void CheckError(OSStatus error, const char *operation)
{
	if (error == noErr) return;
	
	char str[20];
	// see if it appears to be a 4-char-code
	*(UInt32 *)(str + 1) = CFSwapInt32HostToBig(error);
	if (isprint(str[1]) && isprint(str[2]) && isprint(str[3]) && isprint(str[4])) {
		str[0] = str[5] = '\'';
		str[6] = '\0';
	} else
		// no, format it as an integer
		sprintf(str, "%d", (int)error);
	
	fprintf(stderr, "Error: %s (%s)\n", operation, str);
	
	exit(1);
}



@interface MIDIOPlugIn(Execution)
-(void)setupMIDI;
-(void) sendStatus:(Byte)status data1:(Byte)data1 data2:(Byte)data2;
-(void) sendNoteOnEvent:(int)key velocity:(int)velocity ch:(int)ch;
-(void) sendNoteOffEvent:(int)key velocity:(int)velocity ch:(int)ch;
-(NSNumber*)minor_x:(NSNumber*)root index:(NSNumber*)index;
-(NSNumber*)major_x:(NSNumber*)root index:(NSNumber*)index;
-(NSNumber*)minor_pent_x:(NSNumber*)root index:(NSNumber*)index;
-(NSNumber*)major_pent_x:(NSNumber*)root index:(NSNumber*)index;
-(NSNumber*)minor_harmonic_x:(NSNumber*)root index:(NSNumber*)index;
-(NSNumber*)major_harmonic_x:(NSNumber*)root index:(NSNumber*)index;
-(NSNumber*)dorian_x:(NSNumber*)root index:(NSNumber*)index;
-(NSNumber*)mixolydian_x:(NSNumber*)root index:(NSNumber*)index;


@end

@implementation MIDIOPlugIn

// Here you need to declare the input / output properties as dynamic as Quartz Composer will handle their implementation
//@dynamic inputFoo, outputBar;

@dynamic inputDevice;
@dynamic inputCH;
@dynamic inputVelocity;
@dynamic inputTrigger;
@dynamic inputTriggerOff;
@dynamic inputNote;
@dynamic inputIndex;
@dynamic inputScale;
@dynamic inputChord;

+ (NSDictionary *)attributes
{
	// Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary *)attributesForPropertyPortWithKey:(NSString *)key
{
	// Specify the optional attributes for property based ports (QCPortAttributeNameKey, QCPortAttributeDefaultValueKey...).
	return nil;
}

+ (QCPlugInExecutionMode)executionMode
{
	// Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	return kQCPlugInExecutionModeConsumer;
}

+ (QCPlugInTimeMode)timeMode
{
	// Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	return kQCPlugInTimeModeNone;
}

- (id)init
{
	self = [super init];
	if (self) {
		// Allocate any permanent resource required by the plug-in.
	}
	
	return self;
}


@end

@implementation MIDIOPlugIn (Execution)

-(void)setupMIDI{

     NSString * client_string = [NSString stringWithFormat:@"client"];
     CFStringRef cf_client_string = (CFStringRef)client_string ;
     
     NSString * out_string = [NSString stringWithFormat:@"out"];
     CFStringRef cf_out_string = (CFStringRef)out_string ;

     CheckError(MIDIClientCreate(cf_client_string, NULL, NULL, &echo_client),
     "Couldn't create client echo");
     
     CheckError (MIDIOutputPortCreate(echo_client, cf_out_string, &echo_outport),
     "Couldn't out echo port");
     
     echo_destinationEndpoint = MIDIGetDestination(self.inputDevice);
}

-(void) sendStatus:(Byte)status data1:(Byte)data1 data2:(Byte)data2 {
	MIDIPacketList packetList;
	
	packetList.numPackets = 1;
	packetList.packet[0].length = 3;
	packetList.packet[0].data[0] = status;
	packetList.packet[0].data[1] = data1;
	packetList.packet[0].data[2] = data2;
	packetList.packet[0].timeStamp = 0;
	
    
    CheckError( MIDISend(echo_outport, echo_destinationEndpoint, &packetList),
               "Couldn't send MIDI out echo event");
}


-(void) sendNoteOnEvent:(int)key velocity:(int)velocity ch:(int)ch{
    
    [self sendStatus:144+ch data1:(Byte)key & 0x7F data2:(Byte)velocity & 0x7F];
    
}

-(void) sendNoteOffEvent:(int)key velocity:(int)velocity ch:(int)ch{
    
	
    [self sendStatus:128+ch data1:(Byte)key & 0x7F data2:(Byte)velocity & 0x7F];
}


- (BOOL)startExecution:(id <QCPlugInContext>)context
{
    
    [self setupMIDI];
	// Called by Quartz Composer when rendering of the composition starts: perform any required setup for the plug-in.
	// Return NO in case of fatal failure (this will prevent rendering of the composition to start).
	
	return YES;
}

- (void)enableExecution:(id <QCPlugInContext>)context
{
	// Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
}

- (BOOL)execute:(id <QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary *)arguments
{
    
    
    
    
    switch ((int)self.inputScale) {
        case 0:
    if (self.inputTrigger) {
        [self sendNoteOnEvent:[[self minor_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
    }
    if (self.inputTriggerOff) {
        [self sendNoteOnEvent:[[self minor_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:0 ch:(int)self.inputCH];
    }
      break;
        case 1:
            if (self.inputTrigger) {
                [self sendNoteOnEvent:[[self major_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
            }
            if (self.inputTriggerOff) {
                [self sendNoteOnEvent:[[self major_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:0 ch:(int)self.inputCH];
            }
            break;

        case 2:
            if (self.inputTrigger) {
                [self sendNoteOnEvent:[[self minor_harmonic_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
            }
            if (self.inputTriggerOff) {
                [self sendNoteOnEvent:[[self minor_harmonic_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:0 ch:(int)self.inputCH];
            }
            break;

        case 3:
            if (self.inputTrigger) {
                [self sendNoteOnEvent:[[self major_harmonic_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
            }
            if (self.inputTriggerOff) {
                [self sendNoteOnEvent:[[self major_harmonic_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:0 ch:(int)self.inputCH];
            }
            break;

        case 4:
            if (self.inputTrigger) {
                [self sendNoteOnEvent:[[self minor_pent_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
            }
            if (self.inputTriggerOff) {
                [self sendNoteOnEvent:[[self minor_pent_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:0 ch:(int)self.inputCH];
            }
            break;

        case 5:
            if (self.inputTrigger) {
                [self sendNoteOnEvent:[[self major_pent_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
            }
            if (self.inputTriggerOff) {
                [self sendNoteOnEvent:[[self major_pent_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:0 ch:(int)self.inputCH];
            }
            break;

        case 6:
            if (self.inputTrigger) {
                [self sendNoteOnEvent:[[self dorian_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
            }
            if (self.inputTriggerOff) {
                [self sendNoteOnEvent:[[self dorian_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:0 ch:(int)self.inputCH];
            }
            break;
            
        case 7:
            if (self.inputTrigger) {
                [self sendNoteOnEvent:[[self mixolydian_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
            }
            if (self.inputTriggerOff) {
                [self sendNoteOnEvent:[[self mixolydian_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:0 ch:(int)self.inputCH];
            }
            break;
   
    }
    
    ////////////////////////////////////
    ////////////////////////////////////
    //////////////CHORDS/////////
    ///////////////////////////////////
    //////////////////////////////////
    
    if (self.inputChord) {
        switch ((int)self.inputScale) {
            case 0:
                if (self.inputTrigger) {
                    [self sendNoteOnEvent:[[self minor_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                     [self sendNoteOnEvent:[[self minor_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+2]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                     [self sendNoteOnEvent:[[self minor_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+4]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                }
                if (self.inputTriggerOff) {
                    [self sendNoteOnEvent:[[self minor_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:0 ch:(int)self.inputCH];
                      [self sendNoteOnEvent:[[self minor_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+2]] intValue] velocity:0 ch:(int)self.inputCH];
                      [self sendNoteOnEvent:[[self minor_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+4]] intValue] velocity:0 ch:(int)self.inputCH];
                }
                break;
            case 1:
                if (self.inputTrigger) {
                    [self sendNoteOnEvent:[[self major_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                       [self sendNoteOnEvent:[[self major_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+2]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                       [self sendNoteOnEvent:[[self major_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+4]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                }
                if (self.inputTriggerOff) {
                    [self sendNoteOnEvent:[[self major_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:0 ch:(int)self.inputCH];
                      [self sendNoteOnEvent:[[self major_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+2]] intValue] velocity:0 ch:(int)self.inputCH];
                      [self sendNoteOnEvent:[[self major_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+4]] intValue] velocity:0 ch:(int)self.inputCH];
                }
                break;
                
            case 2:
                if (self.inputTrigger) {
                    [self sendNoteOnEvent:[[self minor_harmonic_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                         [self sendNoteOnEvent:[[self minor_harmonic_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+2]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                         [self sendNoteOnEvent:[[self minor_harmonic_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+4]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                }
                if (self.inputTriggerOff) {
                    [self sendNoteOnEvent:[[self minor_harmonic_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:0 ch:(int)self.inputCH];
                     [self sendNoteOnEvent:[[self minor_harmonic_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+2]] intValue] velocity:0 ch:(int)self.inputCH];
                     [self sendNoteOnEvent:[[self minor_harmonic_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+4]] intValue] velocity:0 ch:(int)self.inputCH];
                }
                break;
                
            case 3:
                if (self.inputTrigger) {
                    [self sendNoteOnEvent:[[self major_harmonic_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                       [self sendNoteOnEvent:[[self major_harmonic_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+2]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                       [self sendNoteOnEvent:[[self major_harmonic_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+4]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                }
                if (self.inputTriggerOff) {
                    [self sendNoteOnEvent:[[self major_harmonic_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:0 ch:(int)self.inputCH];
                     [self sendNoteOnEvent:[[self major_harmonic_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+2]] intValue] velocity:0 ch:(int)self.inputCH];
                     [self sendNoteOnEvent:[[self major_harmonic_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+4]] intValue] velocity:0 ch:(int)self.inputCH];
                }
                break;
                
            case 4:
                if (self.inputTrigger) {
                    [self sendNoteOnEvent:[[self minor_pent_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                     [self sendNoteOnEvent:[[self minor_pent_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+2]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                     [self sendNoteOnEvent:[[self minor_pent_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+4]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                    
                }
                if (self.inputTriggerOff) {
                    [self sendNoteOnEvent:[[self minor_pent_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:0 ch:(int)self.inputCH];
                     [self sendNoteOnEvent:[[self minor_pent_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+2]] intValue] velocity:0 ch:(int)self.inputCH];
                     [self sendNoteOnEvent:[[self minor_pent_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+4]] intValue] velocity:0 ch:(int)self.inputCH];
                }
                break;
                
            case 5:
                if (self.inputTrigger) {
                    [self sendNoteOnEvent:[[self major_pent_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                      [self sendNoteOnEvent:[[self major_pent_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+2]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                      [self sendNoteOnEvent:[[self major_pent_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+4]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                }
                if (self.inputTriggerOff) {
                    [self sendNoteOnEvent:[[self major_pent_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:0 ch:(int)self.inputCH];
                    [self sendNoteOnEvent:[[self major_pent_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+2]] intValue] velocity:0 ch:(int)self.inputCH];
                    [self sendNoteOnEvent:[[self major_pent_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+4]] intValue] velocity:0 ch:(int)self.inputCH];
                }
                break;
                
            case 6:
                if (self.inputTrigger) {
                    [self sendNoteOnEvent:[[self dorian_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                      [self sendNoteOnEvent:[[self dorian_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+2]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                      [self sendNoteOnEvent:[[self dorian_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+4]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                }
                if (self.inputTriggerOff) {
                    [self sendNoteOnEvent:[[self dorian_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:0 ch:(int)self.inputCH];
                       [self sendNoteOnEvent:[[self dorian_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+2]] intValue] velocity:0 ch:(int)self.inputCH];
                       [self sendNoteOnEvent:[[self dorian_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+4]] intValue] velocity:0 ch:(int)self.inputCH];
                }
                break;
                
            case 7:
                if (self.inputTrigger) {
                    [self sendNoteOnEvent:[[self mixolydian_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                     [self sendNoteOnEvent:[[self mixolydian_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+2]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                     [self sendNoteOnEvent:[[self mixolydian_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+4]] intValue] velocity:(int)self.inputVelocity ch:(int)self.inputCH];
                }
                if (self.inputTriggerOff) {
                    [self sendNoteOnEvent:[[self mixolydian_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex]] intValue] velocity:0 ch:(int)self.inputCH];
                     [self sendNoteOnEvent:[[self mixolydian_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+2]] intValue] velocity:0 ch:(int)self.inputCH];
                     [self sendNoteOnEvent:[[self mixolydian_x:[NSNumber numberWithInt:(int)self.inputNote] index:[NSNumber numberWithInt:(int)self.inputIndex+4]] intValue] velocity:0 ch:(int)self.inputCH];
                }
                break;
                
        }

    }

    
	
	return YES;
}

- (void)disableExecution:(id <QCPlugInContext>)context
{
	// Called by Quartz Composer when the plug-in instance stops being used by Quartz Composer.
}

- (void)stopExecution:(id <QCPlugInContext>)context
{
  

	// Called by Quartz Composer when rendering of the composition stops: perform any required cleanup for the plug-in.
}



-(NSNumber*)minor_x:(NSNumber*)root index:(NSNumber*)index{
    
    //Natural Minor Scale: R, W, H, W, W, H, W, W
    
    
    switch ([index intValue]) {
        case 0:
            return [NSNumber numberWithInt:[root intValue]];
            break;
        case 1:
            return [NSNumber numberWithInt:[root intValue]+2];
            break;
        case 2:
            return [NSNumber numberWithInt:[root intValue]+2+1];
            break;
        case 3:
            return [NSNumber numberWithInt:[root intValue]+2+1+2];
            break;
        case 4:
            return [NSNumber numberWithInt:[root intValue]+2+1+2+2];
            break;
        case 5:
            return [NSNumber numberWithInt:[root intValue]+2+1+2+2+1];
            break;
        case 6:
            return [NSNumber numberWithInt:[root intValue]+2+1+2+2+1+2];
            break;
        case 7:
            return [NSNumber numberWithInt:[root intValue]+2+1+2+2+1+2+2];
            break;
        case 8:
            return [NSNumber numberWithInt:[root intValue]+12];
            break;
            
            
        default:[NSNumber numberWithInt:[root intValue]];
            break;
    }
    
    
    
    return [NSNumber numberWithInt:[root intValue]];
    
}


-(NSNumber*)major_x:(NSNumber*)root index:(NSNumber*)index
{
    
    //  Major Scale: R, W, W, H, W, W, W, H
    //R, W, W, H, W, W, W, H
    
    switch ([index intValue]) {
        case 0:
            return [NSNumber numberWithInt:[root intValue]];
            break;
        case 1:
            return [NSNumber numberWithInt:[root intValue]+2];
            break;
        case 2:
            return [NSNumber numberWithInt:[root intValue]+2+2];
            break;
        case 3:
            return [NSNumber numberWithInt:[root intValue]+2+2+1];
            break;
        case 4:
            return [NSNumber numberWithInt:[root intValue]+2+2+1+2];
            break;
        case 5:
            return [NSNumber numberWithInt:[root intValue]+2+2+1+2+2];
            break;
        case 6:
            return [NSNumber numberWithInt:[root intValue]+2+2+1+2+2+2];
            break;
        case 7:
            return [NSNumber numberWithInt:[root intValue]+2+2+1+2+2+2+1];
            break;
        case 8:
            return [NSNumber numberWithInt:[root intValue]+12];
            break;
            
            
        default:[NSNumber numberWithInt:[root intValue]];
            break;
    }
    
    
    
    return [NSNumber numberWithInt:[root intValue]];
    
    
    
    
    
    
}

-(NSNumber*)minor_pent_x:(NSNumber*)root index:(NSNumber*)index{
    
    //R, 1 1/2, W, W, 1 1/2, W
    //W + H ... W ... W ... W + H ... W
    
    switch ([index intValue]) {
        case 0:
            return [NSNumber numberWithInt:[root intValue]];
            break;
        case 1:
            return [NSNumber numberWithInt:[root intValue]+3];
            break;
        case 2:
            return [NSNumber numberWithInt:[root intValue]+3+2];
            break;
        case 3:
            return [NSNumber numberWithInt:[root intValue]+3+2+2];
            break;
        case 4:
            return [NSNumber numberWithInt:[root intValue]+3+2+2+3];
            break;
        case 5:
            return [NSNumber numberWithInt:[root intValue]+3+2+2+3+2];
            break;
        case 6:
            return [NSNumber numberWithInt:[root intValue]+12+3];
            break;
        case 7:
            return [NSNumber numberWithInt:[root intValue]+12+3+2];
            break;
        case 8:
            return [NSNumber numberWithInt:[root intValue]+12];
            break;
            
            
        default:[NSNumber numberWithInt:[root intValue]];
            break;
    }
    
    
    
    return [NSNumber numberWithInt:[root intValue]];
    
    
    
    
    
    
    
}
-(NSNumber*)major_pent_x:(NSNumber*)root index:(NSNumber*)index
{
    //W ... W ... W + H ... W ...W + H 
    
    switch ([index intValue]) {
        case 0:
            return [NSNumber numberWithInt:[root intValue]];
            break;
        case 1:
            return [NSNumber numberWithInt:[root intValue]+2];
            break;
        case 2:
            return [NSNumber numberWithInt:[root intValue]+2+2];
            break;
        case 3:
            return [NSNumber numberWithInt:[root intValue]+2+2+2+1];
            break;
        case 4:
            return [NSNumber numberWithInt:[root intValue]+2+2+2+1+2];
            break;
        case 5:
            return [NSNumber numberWithInt:[root intValue]+2+2+2+1+2+2+1];
            break;
        case 6:
            return [NSNumber numberWithInt:[root intValue]+12+2];
            break;
        case 7:
            return [NSNumber numberWithInt:[root intValue]+12+2+2];
            break;
        case 8:
            return [NSNumber numberWithInt:[root intValue]+12];
            break;
            
            
        default:[NSNumber numberWithInt:[root intValue]];
            break;
    }
    
    
    
    return [NSNumber numberWithInt:[root intValue]];
    
    
    
}

-(NSNumber*)minor_harmonic_x:(NSNumber*)root index:(NSNumber*)index{
    
    //root
    // root+2
    //  root+2+1
    //  root+2+1+2
    //   root+2+1+2+2
    //   root+2+1+2+2+1
    //   root+2+1+2+2+1+3
    //   root+2+1+2+2+1+3+1
    
    switch ([index intValue]) {
        case 0:
            return [NSNumber numberWithInt:[root intValue]];
            break;
        case 1:
            return [NSNumber numberWithInt:[root intValue]+2];
            break;
        case 2:
            return [NSNumber numberWithInt:[root intValue]+2+1];
            break;
        case 3:
            return [NSNumber numberWithInt:[root intValue]+2+1+2];
            break;
        case 4:
            return [NSNumber numberWithInt:[root intValue]+2+1+2+2];
            break;
        case 5:
            return [NSNumber numberWithInt:[root intValue]+2+1+2+2+1];
            break;
        case 6:
            return [NSNumber numberWithInt:[root intValue]+2+1+2+2+1+3];
            break;
        case 7:
            return [NSNumber numberWithInt:[root intValue]+2+1+2+2+1+3+1];
            break;
        case 8:
            return [NSNumber numberWithInt:[root intValue]+12];
            break;
            
            
        default:[NSNumber numberWithInt:[root intValue]];
            break;
    }
    
    
    
    return [NSNumber numberWithInt:[root intValue]];
    
    
    
    
}
-(NSNumber*)major_harmonic_x:(NSNumber*)root index:(NSNumber*)index{
    
    //  root
    // root+2
    //  root+2+2
    //   root+2+2+1
    //   root+2+2+1+2
    //   root+2+2+1+2+1
    //   root+2+2+1+2+1+3
    //   root+2+2+1+2+1+3+1
    
    switch ([index intValue]) {
        case 0:
            return [NSNumber numberWithInt:[root intValue]];
            break;
        case 1:
            return [NSNumber numberWithInt:[root intValue]+2];
            break;
        case 2:
            return [NSNumber numberWithInt:[root intValue]+2+2];
            break;
        case 3:
            return [NSNumber numberWithInt:[root intValue]+2+2+1];
            break;
        case 4:
            return [NSNumber numberWithInt:[root intValue]+2+2+1+2];
            break;
        case 5:
            return [NSNumber numberWithInt:[root intValue]+2+2+1+2+1];
            break;
        case 6:
            return [NSNumber numberWithInt:[root intValue]+2+2+1+2+1+3];
            break;
        case 7:
            return [NSNumber numberWithInt:[root intValue]+2+2+1+2+1+3+1];
            break;
        case 8:
            return [NSNumber numberWithInt:[root intValue]+12];
            break;
            
            
        default:[NSNumber numberWithInt:[root intValue]];
            break;
    }
    
    
    
    return [NSNumber numberWithInt:[root intValue]];
    
    
    
    
    
    
    
}

-(NSNumber*)dorian_x:(NSNumber*)root index:(NSNumber*)index{
    
    //R, W, H, W, W, W, H, W
    
    switch ([index intValue]) {
        case 0:
            return [NSNumber numberWithInt:[root intValue]];
            break;
        case 1:
            return [NSNumber numberWithInt:[root intValue]+2];
            break;
        case 2:
            return [NSNumber numberWithInt:[root intValue]+2+1];
            break;
        case 3:
            return [NSNumber numberWithInt:[root intValue]+2+1+2];
            break;
        case 4:
            return [NSNumber numberWithInt:[root intValue]+2+1+2+2];
            break;
        case 5:
            return [NSNumber numberWithInt:[root intValue]+2+1+2+2+2];
            break;
        case 6:
            return [NSNumber numberWithInt:[root intValue]+2+1+2+2+2+1];
            break;
        case 7:
            return [NSNumber numberWithInt:[root intValue]+2+1+2+2+2+1+2];
            break;
        case 8:
            return [NSNumber numberWithInt:[root intValue]+12];
            break;
            
            
        default:[NSNumber numberWithInt:[root intValue]];
            break;
    }
    
    
    
    return [NSNumber numberWithInt:[root intValue]];
    
    
    
}
-(NSNumber*)mixolydian_x:(NSNumber*)root index:(NSNumber*)index{
    
    //  R, W, W, H, W, W, H, W
    
    switch ([index intValue]) {
        case 0:
            return [NSNumber numberWithInt:[root intValue]];
            break;
        case 1:
            return [NSNumber numberWithInt:[root intValue]+2];
            break;
        case 2:
            return [NSNumber numberWithInt:[root intValue]+2+2];
            break;
        case 3:
            return [NSNumber numberWithInt:[root intValue]+2+2+1];
            break;
        case 4:
            return [NSNumber numberWithInt:[root intValue]+2+2+1+2];
            break;
        case 5:
            return [NSNumber numberWithInt:[root intValue]+2+2+1+2+2];
            break;
        case 6:
            return [NSNumber numberWithInt:[root intValue]+2+2+1+2+2+1];
            break;
        case 7:
            return [NSNumber numberWithInt:[root intValue]+2+2+1+2+2+1+2];
            break;
        case 8:
            return [NSNumber numberWithInt:[root intValue]+12];
            break;
            
            
        default:[NSNumber numberWithInt:[root intValue]];
            break;
    }
    
    
    
    return [NSNumber numberWithInt:[root intValue]];
    
    
    
    
    
    
}


@end
